// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


#include "FreeOTFECypherImpl.h"
#include "FreeOTFECypherCAST6_Gladman.h"

#include "FreeOTFEDebug.h"
#include "FreeOTFElib.h"


// Include Gladman library...
#include <std_defs.h>


// =========================================================================
// Cypher driver init function
// devExt - The device extension to be initialized
NTSTATUS
ImpCypherDriverExtDetailsInit(
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;
    int idx = -1;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_ENTER, ("ImpCypherDriverExtDetailsInit\n"));


    // -- POPULATE DRIVER IDENTIFICATION --
    RtlZeroMemory(devExt->DriverTitle, sizeof(devExt->DriverTitle));
    RtlCopyMemory(
                  devExt->DriverTitle,
                  DRIVER_TITLE,
                  strlen(DRIVER_TITLE)
                 );

    devExt->DriverGUID = DRIVER_GUID;
    devExt->DriverVersionID = DRIVER_CYPHER_VERSION;


    // -- POPULATE cyphers SUPPORTED --
    devExt->CypherDetails = ExAllocatePool(
                                         NonPagedPool,
                                         (sizeof(CYPHER) * CYPHERS_SUPPORTED)
                                        );    

    devExt->CypherCount = CYPHERS_SUPPORTED;


    // -- CAST6-128 --
    idx++;
    RtlZeroMemory(devExt->CypherDetails[idx].Title, MAX_CYPHER_TITLE);
    RtlCopyMemory(
                  devExt->CypherDetails[idx].Title,
                  DRIVER_CIPHER_TITLE_CAST6_128,
                  strlen(DRIVER_CIPHER_TITLE_CAST6_128)
                 );

    devExt->CypherDetails[idx].Mode       = CYPHER_MODE_CBC;
    devExt->CypherDetails[idx].KeySize    = 128;
    devExt->CypherDetails[idx].BlockSize  = CAST6_BLOCK_SIZE;
    devExt->CypherDetails[idx].VersionID  = DRIVER_CYPHER_IMPL_VERSION;
    devExt->CypherDetails[idx].CypherGUID = CIPHER_GUID_CAST6_128;


    // -- CAST6-160 --
    idx++;
    RtlZeroMemory(devExt->CypherDetails[idx].Title, MAX_CYPHER_TITLE);
    RtlCopyMemory(
                  devExt->CypherDetails[idx].Title,
                  DRIVER_CIPHER_TITLE_CAST6_160,
                  strlen(DRIVER_CIPHER_TITLE_CAST6_160)
                 );

    devExt->CypherDetails[idx].Mode       = CYPHER_MODE_CBC;
    devExt->CypherDetails[idx].KeySize    = 160;
    devExt->CypherDetails[idx].BlockSize  = CAST6_BLOCK_SIZE;
    devExt->CypherDetails[idx].VersionID  = DRIVER_CYPHER_IMPL_VERSION;
    devExt->CypherDetails[idx].CypherGUID = CIPHER_GUID_CAST6_160;


    // -- CAST6-192 --
    idx++;
    RtlZeroMemory(devExt->CypherDetails[idx].Title, MAX_CYPHER_TITLE);
    RtlCopyMemory(
                  devExt->CypherDetails[idx].Title,
                  DRIVER_CIPHER_TITLE_CAST6_192,
                  strlen(DRIVER_CIPHER_TITLE_CAST6_192)
                 );

    devExt->CypherDetails[idx].Mode       = CYPHER_MODE_CBC;
    devExt->CypherDetails[idx].KeySize    = 192;
    devExt->CypherDetails[idx].BlockSize  = CAST6_BLOCK_SIZE;
    devExt->CypherDetails[idx].VersionID  = DRIVER_CYPHER_IMPL_VERSION;
    devExt->CypherDetails[idx].CypherGUID = CIPHER_GUID_CAST6_192;


    // -- CAST6-224 --
    idx++;
    RtlZeroMemory(devExt->CypherDetails[idx].Title, MAX_CYPHER_TITLE);
    RtlCopyMemory(
                  devExt->CypherDetails[idx].Title,
                  DRIVER_CIPHER_TITLE_CAST6_224,
                  strlen(DRIVER_CIPHER_TITLE_CAST6_224)
                 );

    devExt->CypherDetails[idx].Mode       = CYPHER_MODE_CBC;
    devExt->CypherDetails[idx].KeySize    = 224;
    devExt->CypherDetails[idx].BlockSize  = CAST6_BLOCK_SIZE;
    devExt->CypherDetails[idx].VersionID  = DRIVER_CYPHER_IMPL_VERSION;
    devExt->CypherDetails[idx].CypherGUID = CIPHER_GUID_CAST6_224;


    // -- CAST6-256 --
    idx++;
    RtlZeroMemory(devExt->CypherDetails[idx].Title, MAX_CYPHER_TITLE);
    RtlCopyMemory(
                  devExt->CypherDetails[idx].Title,
                  DRIVER_CIPHER_TITLE_CAST6_256,
                  strlen(DRIVER_CIPHER_TITLE_CAST6_256)
                 );

    devExt->CypherDetails[idx].Mode       = CYPHER_MODE_CBC;
    devExt->CypherDetails[idx].KeySize    = 256;
    devExt->CypherDetails[idx].BlockSize  = CAST6_BLOCK_SIZE;
    devExt->CypherDetails[idx].VersionID  = DRIVER_CYPHER_IMPL_VERSION;
    devExt->CypherDetails[idx].CypherGUID = CIPHER_GUID_CAST6_256;


    DEBUGOUTCYPHERIMPL(DEBUGLEV_EXIT, ("ImpCypherDriverExtDetailsInit\n"));

    return status;
}


// =========================================================================
// Cypher driver cleardown function
// devExt - The device extension to be cleared down
NTSTATUS
ImpCypherDriverExtDetailsCleardown(    
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_ENTER, ("ImpCypherDriverExtDetailsCleardown\n"));

    if (devExt->CypherDetails != NULL)
        {
        ExFreePool(devExt->CypherDetails);
        }

    devExt->CypherDetails = NULL;
    devExt->CypherCount = 0;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_EXIT, ("ImpCypherDriverExtDetailsCleardown\n"));

    return status;
}


// =========================================================================
// Encryption function
// Note: CyphertextLength must be set to the size of the CyphertextData buffer on
//       entry; on exit, this will be set to the size of the buffer used.
NTSTATUS
ImpCypherEncryptData(
    IN      GUID* CypherGUID,
    IN      int KeyLength,  // In bits
    IN      char *Key,
    IN      char *KeyASCII,  // ASCII representation of "Key"
    IN      int IVLength,  // In bits
    IN      char *IV,
    IN      int PlaintextLength,  // In bytes
    IN      char *PlaintextData,
    OUT     char *CyphertextData
)
{
    NTSTATUS status = STATUS_SUCCESS;
    // Null IV in case we're not given an IV
    char zeroIV[FREEOTFE_MAX_CYPHER_BLOCKSIZE];
    char tmpBuffer[(CAST6_BLOCK_SIZE / 8)];
    int i;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_ENTER, ("ImpCypherEncryptData\n"));

    if (!(
          (IsEqualGUID(&CIPHER_GUID_CAST6_128, CypherGUID)) || 
          (IsEqualGUID(&CIPHER_GUID_CAST6_160, CypherGUID)) ||
          (IsEqualGUID(&CIPHER_GUID_CAST6_192, CypherGUID)) ||
          (IsEqualGUID(&CIPHER_GUID_CAST6_224, CypherGUID)) ||
          (IsEqualGUID(&CIPHER_GUID_CAST6_256, CypherGUID))
         ))
        {
        DEBUGOUTCYPHERIMPL(DEBUGLEV_ERROR, ("Unsupported cipher GUID passed in.\n"));
        status = STATUS_INVALID_PARAMETER;
        }
    else
        {
        // Handle situation where a NULL/zero length IV is passed in
        if ( (IVLength == 0) || (IV == NULL) )
            {
            RtlZeroMemory(&zeroIV, sizeof(zeroIV));
            IV = (char*)&zeroIV;
            }


        // Process data using CBC mode
        //
        // CBC Mode ENCRYPT:
        // *) Setup key
        // *) For each block...
        //     *) XOR plaintext with IV
        //     *) Encrypt XOR'd plaintext with key
        //     *) IV = The just encrypted data

        // Note: "enc" specified; we're ENCRYPTING
        // Note: We use a temp buffer (tmpBuffer) because we can't write to
        //       PlaintextData
        set_key((u4byte *)Key, KeyLength);
        for (i=0; i < PlaintextLength; i += (CAST6_BLOCK_SIZE / 8))
            {
            XORBlock(PlaintextData+i, IV, tmpBuffer, (CAST6_BLOCK_SIZE / 8));
            encrypt((u4byte *)tmpBuffer, (u4byte *)(CyphertextData+i));
            IV = CyphertextData+i;
            }

        SecZeroMemory(tmpBuffer, sizeof(tmpBuffer));
        }


    DEBUGOUTCYPHERIMPL(DEBUGLEV_EXIT, ("ImpCypherEncryptData\n"));

    return status;
}


// =========================================================================
// Decryption function
// Note: PlaintextLength must be set to the size of the PlaintextData buffer on
//       entry; on exit, this will be set to the size of the buffer used.
NTSTATUS
ImpCypherDecryptData(
    IN      GUID* CypherGUID,
    IN      int KeyLength,  // In bits
    IN      char *Key,
    IN      char *KeyASCII,  // ASCII representation of "Key"
    IN      int IVLength,  // In bits
    IN      char *IV,
    IN      int CyphertextLength,  // In bytes
    IN      char *CyphertextData,
    OUT     char *PlaintextData
)
{
    NTSTATUS status = STATUS_SUCCESS;
    // Null IV in case we're not given an IV
    char zeroIV[FREEOTFE_MAX_CYPHER_BLOCKSIZE];
    int i;

    DEBUGOUTCYPHERIMPL(DEBUGLEV_ENTER, ("ImpCypherDecryptData\n"));

    if (!(
          (IsEqualGUID(&CIPHER_GUID_CAST6_128, CypherGUID)) || 
          (IsEqualGUID(&CIPHER_GUID_CAST6_160, CypherGUID)) ||
          (IsEqualGUID(&CIPHER_GUID_CAST6_192, CypherGUID)) ||
          (IsEqualGUID(&CIPHER_GUID_CAST6_224, CypherGUID)) ||
          (IsEqualGUID(&CIPHER_GUID_CAST6_256, CypherGUID))
         ))
        {
        DEBUGOUTCYPHERIMPL(DEBUGLEV_ERROR, ("Unsupported cipher GUID passed in.\n"));
        status = STATUS_INVALID_PARAMETER;
        }
    else
        {
        // Handle situation where a NULL/zero length IV is passed in
        if ( (IVLength == 0) || (IV == NULL) )
            {
            RtlZeroMemory(&zeroIV, sizeof(zeroIV));
            IV = (char*)&zeroIV;
            }


        // Process data using CBC mode
        //
        // CBC Mode DECRYPT:
        // *) Setup key
        // *) For each block...
        //     *) Decrypt cyphertext with key
        //     *) XOR output plaintext with IV
        //     *) IV = Cyphertext data

        // Note: "dec" specified; we're DECRYPTING
        set_key((u4byte *)Key, KeyLength);
        for (i=0; i < CyphertextLength; i += (CAST6_BLOCK_SIZE / 8))
            {
            decrypt((u4byte *)(CyphertextData+i), (u4byte *)(PlaintextData+i));
            XORBlock(PlaintextData+i, IV, PlaintextData+i, (CAST6_BLOCK_SIZE / 8));
            IV = CyphertextData+i;
            }

        }


    DEBUGOUTCYPHERIMPL(DEBUGLEV_EXIT, ("ImpCypherDecryptData\n"));

    return status;
}


// =========================================================================
// =========================================================================

