unit FreeOTFEfrmOptions;
// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TfrmOptions = class(TForm)
    pbOK: TButton;
    pbCancel: TButton;
    gbSystemTrayIcon: TGroupBox;
    ckSaveSettings: TCheckBox;
    ckUseSystemTrayIcon: TCheckBox;
    ckMinToIcon: TCheckBox;
    ckCloseToIcon: TCheckBox;
    GroupBox1: TGroupBox;
    ckAllowMultipleInstances: TCheckBox;
    GroupBox2: TGroupBox;
    hkDismount: THotKey;
    ckHotkeyDismount: TCheckBox;
    ckHotkeyDismountEmerg: TCheckBox;
    hkDismountEmerg: THotKey;
    Label1: TLabel;
    Label2: TLabel;
    ckDisplayToolbar: TCheckBox;
    ckDisplayStatusbar: TCheckBox;
    procedure pbOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure pbCancelClick(Sender: TObject);
    procedure ControlChanged(Sender: TObject);
  private
    procedure EnableDisableControls();
  public
    { Public declarations }
  end;

implementation

{$R *.DFM}

uses
  FreeOTFESettings,
  SDUGeneral;

procedure TfrmOptions.pbOKClick(Sender: TObject);
var
  allOK: boolean;
begin
  // If user previously saved their settings, but doesn't want to anymore, warn
  // user to destroy their settings file
  if (
      Settings.OptSaveSettings and
      not(ckSaveSettings.checked)
     ) then
    begin
    MessageDlg(
               'You have opted not to save your settings.'+SDUCRLF+
               SDUCRLF+
               'Any previous settings you had save will remain stored in'+SDUCRLF+
               'an "ini" file in the same directory as this application.'+SDUCRLF+
               SDUCRLF+
               'You will almost certainly want to either manually destroy'+SDUCRLF+
               'this file, or change it''s "SaveSettings" entry to 0.',
               mtWarning,
               [mbOK],
               0
              );
    end;

  Settings.OptSaveSettings := ckSaveSettings.checked;

  // General...
  Settings.OptAllowMultipleInstances := ckAllowMultipleInstances.checked;
  Settings.OptDisplayToolbar := ckDisplayToolbar.checked;
  Settings.OptDisplayStatusbar := ckDisplayStatusbar.checked;

  // System tray icon related...
  Settings.OptSystemTrayIconDisplay := ckUseSystemTrayIcon.checked;
  Settings.OptSystemTrayIconMinTo   := ckMinToIcon.checked;
  Settings.OptSystemTrayIconCloseTo := ckCloseToIcon.checked;

  // Hotkeys...
  Settings.OptHKeyEnableDismount      := ckHotkeyDismount.checked;
  Settings.OptHKeyKeyDismount         := hkDismount.HotKey;
  Settings.OptHKeyEnableDismountEmerg := ckHotkeyDismountEmerg.checked;
  Settings.OptHKeyKeyDismountEmerg    := hkDismountEmerg.HotKey;


  allOK := TRUE;
  if Settings.OptSaveSettings then
    begin
    allOK := Settings.Save();
    end;

  if allOK then
    begin
    ModalResult := mrOK;
    end;

end;


procedure TfrmOptions.FormCreate(Sender: TObject);
begin
  ckSaveSettings.checked := Settings.OptSaveSettings;

  // General...
  ckAllowMultipleInstances.checked := Settings.OptAllowMultipleInstances;
  ckDisplayToolbar.checked := Settings.OptDisplayToolbar;
  ckDisplayStatusbar.checked := Settings.OptDisplayStatusbar;

  // System tray icon related...
  ckUseSystemTrayIcon.checked := Settings.OptSystemTrayIconDisplay;
  ckMinToIcon.checked         := Settings.OptSystemTrayIconMinTo;
  ckCloseToIcon.checked       := Settings.OptSystemTrayIconCloseTo;

  // Hotkeys...
  ckHotkeyDismount.checked      := Settings.OptHKeyEnableDismount;
  hkDismount.HotKey             := Settings.OptHKeyKeyDismount;
  ckHotkeyDismountEmerg.checked := Settings.OptHKeyEnableDismountEmerg;
  hkDismountEmerg.HotKey        := Settings.OptHKeyKeyDismountEmerg;

  EnableDisableControls();

end;

procedure TfrmOptions.EnableDisableControls();
begin
  SDUEnableControl(ckMinToIcon,   ckUseSystemTrayIcon.checked);
  SDUEnableControl(ckCloseToIcon, ckUseSystemTrayIcon.checked);

  SDUEnableControl(hkDismount,      ckHotkeyDismount.checked);
  SDUEnableControl(hkDismountEmerg, ckHotkeyDismountEmerg.checked);

end;

procedure TfrmOptions.pbCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
  
end;

procedure TfrmOptions.ControlChanged(Sender: TObject);
begin
  EnableDisableControls();
  
end;

END.


